"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.analyzeHead = analyzeHead;
var _utils = require("../utils.js");
var _lib = require("./lib.js");
function analyzeHead(path) {
  if (path.node.leadingComments?.some(comment => /react-declassify-disable/.test(comment.value))) {
    // Explicitly disabled
    return;
  }
  if (path.node.leadingComments?.some(comment => comment.type === "CommentBlock" && /^\*/.test(comment.value) && /@abstract/.test(comment.value)) || path.node.abstract) {
    // This is an abstract class to be inherited; do not attempt transformation.
    return;
  }
  const superClass = path.get("superClass");
  if (!superClass.isExpression()) {
    return;
  }
  const superClassRef = (0, _lib.analyzeLibRef)(superClass);
  if (!superClassRef || !(0, _lib.isReactRef)(superClassRef)) {
    return;
  }
  if (superClassRef.name === "Component" || superClassRef.name === "PureComponent") {
    const name = path.node.id;
    const typeParameters_ = (0, _utils.nonNullPath)(path.get("typeParameters"));
    const typeParameters = typeParameters_?.isTSTypeParameterDeclaration() ? typeParameters_ : undefined;
    const isPure = superClassRef.name === "PureComponent";
    let props;
    let propsEach = undefined;
    let states = undefined;
    const superTypeParameters = path.get("superTypeParameters");
    if (superTypeParameters.isTSTypeParameterInstantiation()) {
      const params = superTypeParameters.get("params");
      if (params.length > 0) {
        props = params[0];
        propsEach = decompose(params[0]);
      }
      if (params.length > 1) {
        const stateParamPath = params[1];
        states = decompose(stateParamPath);
      }
    }
    propsEach ??= new Map();
    states ??= new Map();
    return {
      name,
      typeParameters,
      superClassRef,
      isPure,
      props,
      propsEach,
      states
    };
  }
}
function decompose(path) {
  const aliasPath = resolveAlias(path);
  const members = aliasPath.isTSTypeLiteral() ? aliasPath.get("members") : aliasPath.isTSInterfaceBody() ? aliasPath.get("body") : undefined;
  const decomposed = new Map();
  if (members) {
    for (const member of members) {
      if (member.isTSPropertySignature() || member.isTSMethodSignature()) {
        const name = (0, _utils.memberName)(member.node);
        if (name != null) {
          decomposed.set(name, member);
        }
      }
    }
  }
  return decomposed;
}
function resolveAlias(path) {
  if (path.isTSTypeReference()) {
    const typeNamePath = path.get("typeName");
    if (typeNamePath.isIdentifier()) {
      const name = typeNamePath.node.name;
      let scope = typeNamePath.scope;
      while (scope) {
        if (scope.path.isBlockStatement() || scope.path.isProgram()) {
          const path_ = scope.path;
          for (const body of path_.get("body")) {
            if (body.isTSTypeAliasDeclaration() && body.node.id.name === name) {
              return body.get("typeAnnotation");
            } else if (body.isTSInterfaceDeclaration() && body.node.id.name === name) {
              return body.get("body");
            }
          }
        }
        scope = scope.parent;
      }
    }
  }
  return path;
}